# -*- coding: utf8 -*-
from __future__ import unicode_literals

from driverng import Worklist
from driverng.astm import (
    AstmMessageV2,
    HeaderSegment,
    HprimDate,
    HprimForeign,
    HprimString,
    Segment,
)


class KibionDynamicSegmentH(HeaderSegment):
    sender_name = HprimString(5)

    patients = HprimForeign("P", "header")
    queries = HprimForeign("Q", "header")
    end = HprimForeign("L", "header", one2one=True, required=True)


class KibionDynamicSegmentP(Segment):
    patient_id = HprimString(3)
    last_name = HprimString((6, 1))
    first_name = HprimString((6, 2))
    birthdate = HprimDate(8, fmt_date="%Y%m%d")
    sex = HprimString(9)
    height = HprimString(17)
    weight = HprimString(18)

    orders = HprimForeign("O", "patient")


class KibionDynamicSegmentQ(Segment):
    patient_id = HprimString((3, 1))
    sample_id = HprimString((3, 2))


class KibionDynamicSegmentO(Segment):
    sample_id = HprimString((3, 1))
    second_test_delay = HprimString((3, 2))
    order_test_id = HprimString((5, 4))
    order_test_date = HprimDate(7, fmt_date="%Y%m%d")

    results = HprimForeign("R", "order")


class KibionDynamicSegmentR(Segment):
    result_test_id = HprimString((3, 4))
    result_value = HprimString((4, 2))
    result_units = HprimString(5)
    result_status = HprimString(9)


class KibionDynamicSegmentL(Segment):
    code = HprimString(3)


class Driver(AstmMessageV2):
    SPEC = {"code": "kibion_dynamic", "label": "KIBION Dynamic"}

    SEGMENT_CLASS_MAP = {
        "H": KibionDynamicSegmentH,
        "Q": KibionDynamicSegmentQ,
        "P": KibionDynamicSegmentP,
        "O": KibionDynamicSegmentO,
        "R": KibionDynamicSegmentR,
        "L": KibionDynamicSegmentL,
    }

    TYPE_RES = ("12CO2", "13CO2", "Cum.dose", "Delta", "DOB", "Dose/h")

    DEFAULT_SEPARATORS = "|\\^&"

    OPTIONS = [
        AstmMessageV2.Option(
            "kibion_dynamic_sender_name", label="Sender Name", default=""
        )
    ]

    def __init__(self, **kwargs):
        self.kibion_dynamic_sender_name = kwargs.pop("kibion_dynamic_sender_name", "")
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        message = KibionDynamicSegmentH(type_segment="H", **kwargs)
        message.end = KibionDynamicSegmentL(type_segment="L")
        return message

    def encode_worklist(self, worklist, context):
        if worklist.context != "order":
            raise NotImplementedError("context %s" % (worklist.context,))
        msg = self.create(date=context["now"])
        msg.set_raw("sender_name", self.kibion_dynamic_sender_name)
        for patient in worklist.get_astm_worklist(
            unique_together=["action", "priority"],
            priority={"S": "S", worklist.ELSE: "R"},
            action={"del": "C", "add": "A", "new": "A"},
        ):

            seg_p = msg.patients.add(
                patient_id=patient.patient_id,
                last_name=patient.last_name,
                first_name=patient.first_name,
                birthdate=patient.birthdate,
                sex=patient.sex,
                height="{:.0f}".format(float(patient.height) * 100)
                if patient.height
                else "",
                weight=patient.weight,
            )

            for sample, action, priority in patient.samples:
                seg_o = seg_p.orders.add(
                    sample_id=sample.sample_id,
                    priority=priority,
                    collection_date=sample.collection_date,
                    action_code=action,
                )
                for order in sample.orders:
                    seg_o.order_test_id = order.code

        if msg.patients:
            return msg
        elif context["query"] and context["query"].queries:
            msg.end.terminaison_code = "I"
            return msg

    def decode_message(self, message):
        if message.queries:
            worklist = Worklist(context="query")
            for seg_q in message.queries:
                worklist.get_or_create(type="patient", sample_id=seg_q.sample_id)
        else:
            worklist = Worklist(context="result")
            for seg_p in message.patients:
                for seg_o in seg_p.orders:
                    rang_res = 0
                    sample = worklist.get_or_create(
                        type="patient", sample_id=seg_o.sample_id
                    )

                    for seg_r in seg_o.results:
                        sample.results.add(
                            code="{}-{}-{}".format(
                                seg_r.result_test_id,
                                self.TYPE_RES[rang_res],
                                seg_o.second_test_delay,
                            ),
                            value=seg_r.result_value,
                            unit=seg_r.result_units,
                        )
                        rang_res += 1

        return worklist
