# -*- coding: utf8 -*-
from __future__ import unicode_literals

import datetime
import logging

from driverng import Worklist
from driverng.astm import AstmMessageV2, HeaderSegment, HprimDate
from driverng.astm import HprimForeign as HL7Foreign
from driverng.astm import HprimList as HL7List
from driverng.astm import HprimString as HL7String
from driverng.astm import Segment

logger = logging.getLogger("driverng.starlet")


class HL7Date(HprimDate):
    def to_hprim(self, value):
        return value.strftime("%Y%m%d%H%M%S")

    def to_python(self, value):
        try:
            return datetime.datetime.strptime(value.split("+")[0], "%Y%m%d%H%M%S.%f")
        except ValueError:
            return datetime.datetime.strptime(value.split("+")[0], "%Y%m%d%H%M%S")


class StarletSegmentMSH(HeaderSegment):
    sending_application = HL7String(3, default="MCA")
    sending_facility = HL7String(4, default="MCA")
    receiving_application = HL7String(5, default="Seegene Viewer")
    receiving_facility = HL7String(6, default="Seegene")
    date = HL7Date(7)
    message_type = HL7String((9, 1), default="ORU")
    message_code = HL7String((9, 2), default="R30")
    message_version = HL7String((9, 3))
    version = HL7String(12, default="2.5")

    patients = HL7Foreign("PID", "header")
    orders = HL7Foreign("OBR", "header")
    common_order = HL7Foreign("ORC", "header")


class StarletSegmentPID(Segment):
    external_patient_id = HL7String(2)
    internal_patient_id = HL7String(3)
    alternate_patient_id = HL7String(4)
    first_name = HL7String((6, 1))
    last_name = HL7String((6, 2))

    orders = HL7Foreign("OBR", "patient")
    info_action = HL7Foreign("ORC", "patient")
    infos = HL7Foreign("NTE", "patient")

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(StarletSegmentPID, self).to_string(separators=separators)


class StarletSegmentORC(Segment):
    order_control = HL7String(1)

    orders = HL7Foreign("OBR", "patient")

    def to_string(self, separators=None):
        self.rang_segment = "NW"
        return super(StarletSegmentORC, self).to_string(separators=separators)


class StarletSegmentOBR(Segment):
    well_name = HL7String((4, 1))
    well_id = HL7String((4, 2))
    product_name = HL7String((5, 1))
    auto_interpretation = HL7String((5, 2))
    comment = HL7String((5, 3))

    common_order = HL7Foreign("ORC", "order")
    infos = HL7Foreign("NTE", "order")

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(StarletSegmentOBR, self).to_string(separators=separators)


class StarletSegmentOBX(Segment):
    type_message = HL7String(3)
    lot = HL7String(4)


class StarletSegmentNTE(Segment):
    comment = HL7String(3)
    # Ancienne version du protocole : 5 sous-champs
    # dye~target~decision~result_type~ct~result_value
    # Révision 1.1 du protocole : 6 sous-champs
    # dye~target~code~decision~result_type~ct~result_value
    result = HL7List(4)

    results = HL7Foreign("OBX", "orders")

    def to_string(self, separators=None):
        self.rang_segment = ""
        return super(StarletSegmentNTE, self).to_string(separators=separators)


class Driver(AstmMessageV2):
    SPEC = {"code": "starlet", "label": "Starlet/Nimbus Résultats CFX-96 Seegene HL7"}
    SEGMENT_CLASS_MAP = {
        "MSH": StarletSegmentMSH,
        "PID": StarletSegmentPID,
        "ORC": StarletSegmentORC,
        "OBR": StarletSegmentOBR,
        "OBX": StarletSegmentOBX,
        "NTE": StarletSegmentNTE,
    }
    DEFAULT_SEPARATORS = "|^~\\&"
    FIRST_SEP_INDEX = 3
    SEGMENT_ENDING = "\r"
    OPTIONS = [
        AstmMessageV2.BooleanOption(
            "close_after_read",
            label="Fermeture de la socket à chaque lecture",
            default=False,
        ),
        AstmMessageV2.BooleanOption(
            "starlet_include_results_na",
            label="Inclure les résultats de type N/A",
            default=False,
        ),
    ]

    def __init__(self, **kwargs):
        self.include_results_na = kwargs.pop("starlet_include_results_na", False)
        super(Driver, self).__init__(**kwargs)

    def create(self, **kwargs):
        return StarletSegmentMSH(type_segment="MSH", **kwargs)

    def to_string(self, message, separators=None, **kwargs):
        return super(Driver, self).to_string(message, separators, **kwargs) + "\r"

    @staticmethod
    def get_obr_multi_results(segment, sample, date):
        product_names = segment.product_name.split(" ")
        codes = product_names[1].split("/")
        interpretations = segment.auto_interpretation.split(",")
        results = []
        for i in interpretations:
            results.append(i.replace(" ", ""))
        for code in codes:
            if code in results:
                sample.results.add(code=code, value="Positif", date=date)
            elif "Invalid" in results[0]:
                sample.results.add(code=code, value="Invalide", date=date)
            else:
                sample.results.add(code=code, value="Négatif", date=date)

    def encode_worklist(self, worklist, context):
        msg = self.create(date=context["now"])
        for patient in worklist.get_astm_worklist(
            unique_together=["action"],
            action={"del": worklist.IGNORE, worklist.ELSE: None},
        ):
            for sample, action in patient.samples:
                seg_pid = msg.patients.add(
                    internal_patient_id=sample.sample_id,
                    last_name=patient.last_name,
                    first_name=patient.first_name,
                )
                seg_orc = seg_pid.info_action.add()
                seg_obr = seg_orc.orders.add()
                seg_obr.product_name = sample.sample_id
                order_code = []
                for order in sample["orders"]:
                    order_code.append(order.code)
                if order_code:
                    seg_obr.infos.add(result=order_code)
        if msg.patients:
            return msg

    def decode_message(self, message):
        if message.message_type != "ORU":
            raise NotImplementedError

        worklist = Worklist(context="result")
        # Selon les configurations les résultats du STARLET ne sont pas envoyés
        # de la même manière
        for seg_pid in message.patients:
            if seg_pid.info_action:
                patient_orders = seg_pid.info_action[0].orders
            else:
                patient_orders = seg_pid.orders
            for seg_obr in patient_orders:
                if "Control" in seg_obr.auto_interpretation:
                    sample = worklist.get_or_create(
                        sample_id=seg_obr.well_name,
                        control_lot=seg_obr.well_name,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_obr.well_name,
                        rack=seg_obr.well_id,
                        type="patient",
                    )
                if seg_obr.auto_interpretation:
                    sample.results.add(
                        code="CONCLUSION_GENERALE",
                        value=seg_obr.auto_interpretation,
                        date=message.date,
                    )
                if sample.type == "patient" and seg_obr.well_id:
                    sample.results.add(
                        code="POSITION_CFX", value=seg_obr.well_id, date=message.date
                    )
                if (
                    "/" in seg_obr.product_name
                    and "Control" not in seg_obr.auto_interpretation
                ):
                    self.get_obr_multi_results(seg_obr, sample, message.date)
                for seg_nte in seg_obr.infos:
                    if (
                        len(seg_nte.result) == 5
                        or len(seg_nte.result) == 6
                        and not seg_nte.result[2]
                    ):
                        code = seg_nte.result[1]
                    elif len(seg_nte.result) == 6 and seg_nte.result[2]:
                        code = seg_nte.result[2]
                    else:
                        continue
                    if code and seg_nte.result[3]:
                        sample.results.add(
                            code=code, value=seg_nte.result[3], date=message.date
                        )
                    # LEVEL RESULT
                    if (
                        code
                        and seg_nte.result[5]
                        and not self.include_results_na
                        and seg_nte.result[5] != "N/A"
                    ):
                        sample.results.add(
                            code="LEVEL_{}".format(code),
                            value=seg_nte.result[5],
                            date=message.date,
                        )
                    elif self.include_results_na and code and seg_nte.result[5]:
                        sample.results.add(
                            code="LEVEL_{}".format(code),
                            value=seg_nte.result[5],
                            date=message.date,
                        )

        for seg_obr in message.orders:
            if "Control" in seg_obr.auto_interpretation:
                sample = worklist.get_or_create(
                    sample_id=seg_obr.well_name,
                    control_lot=seg_obr.well_name,
                    type="control",
                )
            else:
                sample = worklist.get_or_create(
                    sample_id=seg_obr.well_name, rack=seg_obr.well_id, type="patient"
                )
            if seg_obr.auto_interpretation:
                sample.results.add(
                    code="CONCLUSION_GENERALE",
                    value=seg_obr.auto_interpretation,
                    date=message.date,
                )
            if sample.type == "patient" and seg_obr.well_id:
                sample.results.add(
                    code="POSITION_CFX", value=seg_obr.well_id, date=message.date
                )
            if (
                "/" in seg_obr.product_name
                and "Control" not in seg_obr.auto_interpretation
            ):
                self.get_obr_multi_results(seg_obr, sample, message.date)
            for seg_nte in seg_obr.infos:
                if (
                    len(seg_nte.result) == 5
                    or len(seg_nte.result) == 6
                    and not seg_nte.result[2]
                ):
                    code = seg_nte.result[1]
                elif len(seg_nte.result) == 6 and seg_nte.result[2]:
                    code = seg_nte.result[2]
                else:
                    continue
                if code and seg_nte.result[3]:
                    sample.results.add(
                        code=code, value=seg_nte.result[3], date=message.date
                    )
                # LEVEL RESULT
                if (
                    code
                    and seg_nte.result[5]
                    and not self.include_results_na
                    and seg_nte.result[5] != "N/A"
                ):
                    sample.results.add(
                        code="LEVEL_{}".format(code),
                        value=seg_nte.result[5],
                        date=message.date,
                    )
                elif self.include_results_na and code and seg_nte.result[5]:
                    sample.results.add(
                        code="LEVEL_{}".format(code),
                        value=seg_nte.result[5],
                        date=message.date,
                    )

        for seg_orc in message.common_order:
            for seg_obr in seg_orc.orders:
                if "Control" in seg_obr.auto_interpretation:
                    sample = worklist.get_or_create(
                        sample_id=seg_obr.well_name,
                        control_lot=seg_obr.well_name,
                        type="control",
                    )
                else:
                    sample = worklist.get_or_create(
                        sample_id=seg_obr.well_name,
                        rack=seg_obr.well_id,
                        type="patient",
                    )
                if sample.type == "patient" and seg_obr.well_id:
                    sample.results.add(
                        code="POSITION_CFX", value=seg_obr.well_id, date=message.date
                    )
                if (
                    "/" in seg_obr.product_name
                    and "Control" not in seg_obr.auto_interpretation
                ):
                    self.get_obr_multi_results(seg_obr, sample, message.date)
                for seg_nte in seg_obr.infos:
                    if (
                        len(seg_nte.result) == 5
                        or len(seg_nte.result) == 6
                        and not seg_nte.result[2]
                    ):
                        code = seg_nte.result[1]
                    elif len(seg_nte.result) == 6 and seg_nte.result[2]:
                        code = seg_nte.result[2]
                    else:
                        continue
                    if code and seg_nte.result[3]:
                        sample.results.add(
                            code=code, value=seg_nte.result[3], date=message.date
                        )
                    # LEVEL RESULT
                    if (
                        code
                        and seg_nte.result[5]
                        and not self.include_results_na
                        and seg_nte.result[5] != "N/A"
                    ):
                        sample.results.add(
                            code="LEVEL_{}".format(code),
                            value=seg_nte.result[5],
                            date=message.date,
                        )
                    elif self.include_results_na and code and seg_nte.result[5]:
                        sample.results.add(
                            code="LEVEL_{}".format(code),
                            value=seg_nte.result[5],
                            date=message.date,
                        )
        return worklist
